/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

/**
 * This is a temporary, non-standard interface, to ease the transition to a
 * world where Attr no longer inherits from Node.
 */

[uuid(cb5564cd-26ec-418f-a6d6-1d57cd2c971c)]
interface nsIDOMMozNamedAttrMap : nsISupports
{
  nsIDOMAttr                getNamedItem(in DOMString name);
  nsIDOMAttr                setNamedItem(in nsIDOMAttr arg)
                                  raises(DOMException);
  nsIDOMAttr                removeNamedItem(in DOMString name)
                                  raises(DOMException);
  nsIDOMAttr                item(in unsigned long index);
  readonly attribute  unsigned long        length;
  // Introduced in DOM Level 2:
  nsIDOMAttr                getNamedItemNS(in DOMString namespaceURI, 
                                           in DOMString localName);
  // Introduced in DOM Level 2:
  nsIDOMAttr                setNamedItemNS(in nsIDOMAttr arg)
                                  raises(DOMException);
  // Introduced in DOM Level 2:
  nsIDOMAttr                removeNamedItemNS(in DOMString namespaceURI, 
                                              in DOMString localName)
                                  raises(DOMException);
};
