/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/*
  For parsing JSON from http://httpwg.org/http-extensions/opsec.html
*/

#include "nsISupports.idl"

%{C++
#define NS_WELLKNOWNOPPORTUNISTICUTILS_CONTRACTID "@mozilla.org/network/well-known-opportunistic-utils;1"
%}

[scriptable, uuid(b4f96c89-5238-450c-8bda-e12c26f1d150)]
interface nsIWellKnownOpportunisticUtils : nsISupports
{
    void verify(in ACString aJSON,
               in ACString  aOrigin,
               in long      aAlternatePort);

   readonly attribute bool valid;
   readonly attribute bool mixed; /* mixed-scheme */
   readonly attribute long lifetime;
};
