/* THIS FILE IS AUTOGENERATED FROM FlyWebDiscoveryManager.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_FlyWebDiscoveryManagerBinding_h
#define mozilla_dom_FlyWebDiscoveryManagerBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/CallbackInterface.h"
#include "mozilla/dom/Nullable.h"
#include "mozilla/dom/ToJSValue.h"

namespace mozilla {
namespace dom {

struct FlyWebDiscoveredService;
struct FlyWebDiscoveredServiceAtoms;
class FlyWebDiscoveryCallback;
struct FlyWebDiscoveryCallbackAtoms;
class FlyWebDiscoveryManager;
struct FlyWebDiscoveryManagerAtoms;
struct FlyWebPairedService;
struct FlyWebPairedServiceAtoms;
class FlyWebPairingCallback;
struct FlyWebPairingCallbackAtoms;
struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct FlyWebDiscoveredService : public DictionaryBase
{
  MOZ_INIT_OUTSIDE_CTOR nsString mCert;
  MOZ_INIT_OUTSIDE_CTOR nsString mDisplayName;
  MOZ_INIT_OUTSIDE_CTOR nsString mPath;
  MOZ_INIT_OUTSIDE_CTOR nsString mServiceId;
  MOZ_INIT_OUTSIDE_CTOR nsString mServiceType;
  MOZ_INIT_OUTSIDE_CTOR nsString mTransport;

  FlyWebDiscoveredService();

  explicit inline FlyWebDiscoveredService(const FastDictionaryInitializer& )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline FlyWebDiscoveredService(const FlyWebDiscoveredService& aOther)
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const FlyWebDiscoveredService& aOther);

private:
  static bool
  InitIds(JSContext* cx, FlyWebDiscoveredServiceAtoms* atomsCache);
};

namespace binding_detail {
struct FastFlyWebDiscoveredService : public FlyWebDiscoveredService
{
  inline FastFlyWebDiscoveredService()
    : FlyWebDiscoveredService(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


struct FlyWebPairedService : public DictionaryBase
{
  MOZ_INIT_OUTSIDE_CTOR FlyWebDiscoveredService mDiscoveredService;
  MOZ_INIT_OUTSIDE_CTOR nsString mHostname;
  MOZ_INIT_OUTSIDE_CTOR nsString mUiUrl;

  FlyWebPairedService();

  explicit inline FlyWebPairedService(const FastDictionaryInitializer& )
    : mDiscoveredService(FastDictionaryInitializer())
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline FlyWebPairedService(const FlyWebPairedService& aOther)
    : mDiscoveredService(FastDictionaryInitializer())
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const FlyWebPairedService& aOther);

private:
  static bool
  InitIds(JSContext* cx, FlyWebPairedServiceAtoms* atomsCache);
};

namespace binding_detail {
struct FastFlyWebPairedService : public FlyWebPairedService
{
  inline FastFlyWebPairedService()
    : FlyWebPairedService(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


namespace FlyWebDiscoveryManagerBinding {

  typedef mozilla::dom::FlyWebDiscoveryManager NativeType;

  bool
  ConstructorEnabled(JSContext* aCx, JS::Handle<JSObject*> aObj);

  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  const JSClass*
  GetJSClass();

  bool
  Wrap(JSContext* aCx, mozilla::dom::FlyWebDiscoveryManager* aObject, nsWrapperCache* aCache, JS::Handle<JSObject*> aGivenProto, JS::MutableHandle<JSObject*> aReflector);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject, JS::Handle<JSObject*> aGivenProto)
  {
    JS::Rooted<JSObject*> reflector(aCx);
    return Wrap(aCx, aObject, aObject, aGivenProto, &reflector) ? reflector.get() : nullptr;
  }

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObjectHandle(JSContext* aCx);

  JS::Handle<JSObject*>
  GetConstructorObjectHandle(JSContext* aCx, bool aDefineOnGlobal = true);

  JSObject*
  GetConstructorObject(JSContext* aCx);

} // namespace FlyWebDiscoveryManagerBinding



class FlyWebDiscoveryCallback : public CallbackInterface
{
public:
  explicit inline FlyWebDiscoveryCallback(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCx, aCallback, aIncumbentGlobal)
  {
  }

  explicit inline FlyWebDiscoveryCallback(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal, const FastCallbackConstructor& )
    : CallbackInterface(aCx, aCallback, aIncumbentGlobal, FastCallbackConstructor())
  {
  }

  explicit inline FlyWebDiscoveryCallback(JS::Handle<JSObject*> aCallback, JS::Handle<JSObject*> aAsyncStack, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCallback, aAsyncStack, aIncumbentGlobal)
  {
  }

  template <typename T>
  inline void
  OnDiscoveredServicesChanged(const T& thisVal, const Sequence<FlyWebDiscoveredService>& serviceList, ErrorResult& aRv, const char* aExecutionReason = nullptr, ExceptionHandling aExceptionHandling = eReportExceptions, JSCompartment* aCompartment = nullptr)
  {
    if (!aExecutionReason) {
      aExecutionReason = "FlyWebDiscoveryCallback.onDiscoveredServicesChanged";
    }
    CallSetup s(this, aRv, aExecutionReason, aExceptionHandling, aCompartment);
    if (!s.GetContext()) {
      MOZ_ASSERT(aRv.Failed());
      return;
    }
    JS::Rooted<JS::Value> thisValJS(s.GetContext());
    if (!ToJSValue(s.GetContext(), thisVal, &thisValJS)) {
      aRv.Throw(NS_ERROR_FAILURE);
      return;
    }
    return OnDiscoveredServicesChanged(s.GetContext(), thisValJS, serviceList, aRv);
  }

  inline void
  OnDiscoveredServicesChanged(const Sequence<FlyWebDiscoveredService>& serviceList, ErrorResult& aRv, const char* aExecutionReason = nullptr, ExceptionHandling aExceptionHandling = eReportExceptions, JSCompartment* aCompartment = nullptr)
  {
    if (!aExecutionReason) {
      aExecutionReason = "FlyWebDiscoveryCallback.onDiscoveredServicesChanged";
    }
    CallSetup s(this, aRv, aExecutionReason, aExceptionHandling, aCompartment);
    if (!s.GetContext()) {
      MOZ_ASSERT(aRv.Failed());
      return;
    }
    return OnDiscoveredServicesChanged(s.GetContext(), JS::UndefinedHandleValue, serviceList, aRv);
  }

  template <typename T>
  inline void
  OnDiscoveredServicesChanged(const T& thisVal, const Sequence<FlyWebDiscoveredService>& serviceList, const char* aExecutionReason = nullptr)
  {
    IgnoredErrorResult rv;
    return OnDiscoveredServicesChanged(thisVal, serviceList, rv, aExecutionReason);
  }

  inline void
  OnDiscoveredServicesChanged(const Sequence<FlyWebDiscoveredService>& serviceList, const char* aExecutionReason = nullptr)
  {
    IgnoredErrorResult rv;
    return OnDiscoveredServicesChanged(serviceList, rv, aExecutionReason, eReportExceptions, nullptr);
  }

  inline bool
  operator==(const FlyWebDiscoveryCallback& aOther) const
  {
    return CallbackInterface::operator==(aOther);
  }

private:
  void OnDiscoveredServicesChanged(JSContext* cx, JS::Handle<JS::Value> aThisVal, const Sequence<FlyWebDiscoveredService>& serviceList, ErrorResult& aRv);

  static bool
  InitIds(JSContext* cx, FlyWebDiscoveryCallbackAtoms* atomsCache);
};


class FlyWebPairingCallback : public CallbackInterface
{
public:
  explicit inline FlyWebPairingCallback(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCx, aCallback, aIncumbentGlobal)
  {
  }

  explicit inline FlyWebPairingCallback(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal, const FastCallbackConstructor& )
    : CallbackInterface(aCx, aCallback, aIncumbentGlobal, FastCallbackConstructor())
  {
  }

  explicit inline FlyWebPairingCallback(JS::Handle<JSObject*> aCallback, JS::Handle<JSObject*> aAsyncStack, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCallback, aAsyncStack, aIncumbentGlobal)
  {
  }

  void PairingSucceeded(const FlyWebPairedService& service, ErrorResult& aRv, const char* aExecutionReason = nullptr, ExceptionHandling aExceptionHandling = eReportExceptions, JSCompartment* aCompartment = nullptr);

  void PairingFailed(const nsAString& error, ErrorResult& aRv, const char* aExecutionReason = nullptr, ExceptionHandling aExceptionHandling = eReportExceptions, JSCompartment* aCompartment = nullptr);

  inline bool
  operator==(const FlyWebPairingCallback& aOther) const
  {
    return CallbackInterface::operator==(aOther);
  }

private:
  static bool
  InitIds(JSContext* cx, FlyWebPairingCallbackAtoms* atomsCache);
};


namespace binding_detail {
class FastFlyWebDiscoveryCallback : public FlyWebDiscoveryCallback
{
public:
  explicit inline FastFlyWebDiscoveryCallback(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : FlyWebDiscoveryCallback(aCx, aCallback, aIncumbentGlobal, FastCallbackConstructor())
  {
  }

  inline void
  Trace(JSTracer* aTracer)
  {
    FlyWebDiscoveryCallback::Trace(aTracer);
  }

  inline void
  HoldJSObjectsIfMoreThanOneOwner()
  {
    FlyWebDiscoveryCallback::HoldJSObjectsIfMoreThanOneOwner();
  }
};
} // namespace binding_detail


namespace binding_detail {
class FastFlyWebPairingCallback : public FlyWebPairingCallback
{
public:
  explicit inline FastFlyWebPairingCallback(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : FlyWebPairingCallback(aCx, aCallback, aIncumbentGlobal, FastCallbackConstructor())
  {
  }

  inline void
  Trace(JSTracer* aTracer)
  {
    FlyWebPairingCallback::Trace(aTracer);
  }

  inline void
  HoldJSObjectsIfMoreThanOneOwner()
  {
    FlyWebPairingCallback::HoldJSObjectsIfMoreThanOneOwner();
  }
};
} // namespace binding_detail


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_FlyWebDiscoveryManagerBinding_h
