/* THIS FILE IS AUTOGENERATED FROM InspectorUtils.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_InspectorUtilsBinding_h
#define mozilla_dom_InspectorUtilsBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct InspectorRGBATupleAtoms;
struct InspectorRGBTripleAtoms;
struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct InspectorRGBATuple : public DictionaryBase
{
  MOZ_INIT_OUTSIDE_CTOR double mA;
  MOZ_INIT_OUTSIDE_CTOR double mB;
  MOZ_INIT_OUTSIDE_CTOR double mG;
  MOZ_INIT_OUTSIDE_CTOR double mR;

  InspectorRGBATuple();

  explicit inline InspectorRGBATuple(const FastDictionaryInitializer& )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline InspectorRGBATuple(const InspectorRGBATuple& aOther)
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const InspectorRGBATuple& aOther);

private:
  static bool
  InitIds(JSContext* cx, InspectorRGBATupleAtoms* atomsCache);
};

namespace binding_detail {
struct FastInspectorRGBATuple : public InspectorRGBATuple
{
  inline FastInspectorRGBATuple()
    : InspectorRGBATuple(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


struct InspectorRGBTriple : public DictionaryBase
{
  MOZ_INIT_OUTSIDE_CTOR uint8_t mB;
  MOZ_INIT_OUTSIDE_CTOR uint8_t mG;
  MOZ_INIT_OUTSIDE_CTOR uint8_t mR;

  InspectorRGBTriple();

  explicit inline InspectorRGBTriple(const FastDictionaryInitializer& )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline InspectorRGBTriple(const InspectorRGBTriple& aOther)
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const InspectorRGBTriple& aOther);

private:
  static bool
  InitIds(JSContext* cx, InspectorRGBTripleAtoms* atomsCache);
};

namespace binding_detail {
struct FastInspectorRGBTriple : public InspectorRGBTriple
{
  inline FastInspectorRGBTriple()
    : InspectorRGBTriple(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_InspectorUtilsBinding_h
