/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include "nsISupports.idl"

/**
 * Listener to handle cross partial nsISHistory navigation requests.
 */
[scriptable, uuid(be0cd2b6-6f03-4366-9fe2-184c914ff3df)]
interface nsIPartialSHistoryListener : nsISupports
{
  /**
   * Called when the navigation target belongs to another nsISHistory within
   * the same nsIGroupedSHistory, and it needs to initiate cross nsISHistory
   * navigation.
   *
   * @param aIndex The index of complete history to navigate to.
   */
   void onRequestCrossBrowserNavigation(in unsigned long aIndex);
};
