/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIURI;
interface nsIDOMNode;
interface nsISimpleEnumerator;

[scriptable, uuid(422a1807-4e7f-463d-b8d7-ca2ceb9b5d53)]
interface nsIPrefetchService : nsISupports
{
    /**
     * Enqueue a request to prefetch the specified URI.
     *
     * @param aURI the URI of the document to prefetch
     * @param aReferrerURI the URI of the referring page
     * @param aSource the DOM node (such as a <link> tag) that requested this
     *        fetch, or null if the prefetch was not requested by a DOM node.
     * @param aExplicit the link element has an explicit prefetch link type
     */
    void prefetchURI(in nsIURI aURI,
                     in nsIURI aReferrerURI,
                     in nsIDOMNode aSource,
                     in boolean aExplicit);

    /**
     * Find out if there are any prefetches running or queued
     */
    boolean hasMoreElements();

    /**
     * Cancel prefetch
     */
    void cancelPrefetchURI(in nsIURI aURI, in nsIDOMNode aSource);
};
