/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIAppShellService.idl
 */

#ifndef __gen_nsIAppShellService_h__
#define __gen_nsIAppShellService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIXULWindow; /* forward declaration */

class nsIWindowlessBrowser; /* forward declaration */

class nsIURI; /* forward declaration */

class mozIDOMWindowProxy; /* forward declaration */

class nsIAppShell; /* forward declaration */

class nsITabParent; /* forward declaration */

#include "js/TypeDecls.h"

/* starting interface:    nsIAppShellService */
#define NS_IAPPSHELLSERVICE_IID_STR "19266025-354c-4bb9-986b-3483b2b1cdef"

#define NS_IAPPSHELLSERVICE_IID \
  {0x19266025, 0x354c, 0x4bb9, \
    { 0x98, 0x6b, 0x34, 0x83, 0xb2, 0xb1, 0xcd, 0xef }}

class NS_NO_VTABLE nsIAppShellService : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IAPPSHELLSERVICE_IID)

  enum {
    SIZE_TO_CONTENT = -1
  };

  /* nsIXULWindow createTopLevelWindow (in nsIXULWindow aParent, in nsIURI aUrl, in uint32_t aChromeMask, in long aInitialWidth, in long aInitialHeight, in nsITabParent aOpeningTab, in mozIDOMWindowProxy aOpenerWindow); */
  NS_IMETHOD CreateTopLevelWindow(nsIXULWindow *aParent, nsIURI *aUrl, uint32_t aChromeMask, int32_t aInitialWidth, int32_t aInitialHeight, nsITabParent *aOpeningTab, mozIDOMWindowProxy *aOpenerWindow, nsIXULWindow * *_retval) = 0;

  /* nsIWindowlessBrowser createWindowlessBrowser ([optional] in bool aIsChrome); */
  NS_IMETHOD CreateWindowlessBrowser(bool aIsChrome, nsIWindowlessBrowser * *_retval) = 0;

  /* [noscript] void createHiddenWindow (); */
  NS_IMETHOD CreateHiddenWindow(void) = 0;

  /* void destroyHiddenWindow (); */
  NS_IMETHOD DestroyHiddenWindow(void) = 0;

  /* [noscript] void setScreenId (in uint32_t aScreenId); */
  NS_IMETHOD SetScreenId(uint32_t aScreenId) = 0;

  /* readonly attribute nsIXULWindow hiddenWindow; */
  NS_IMETHOD GetHiddenWindow(nsIXULWindow * *aHiddenWindow) = 0;

  /* readonly attribute mozIDOMWindowProxy hiddenDOMWindow; */
  NS_IMETHOD GetHiddenDOMWindow(mozIDOMWindowProxy * *aHiddenDOMWindow) = 0;

  /* readonly attribute nsIXULWindow hiddenPrivateWindow; */
  NS_IMETHOD GetHiddenPrivateWindow(nsIXULWindow * *aHiddenPrivateWindow) = 0;

  /* readonly attribute mozIDOMWindowProxy hiddenPrivateDOMWindow; */
  NS_IMETHOD GetHiddenPrivateDOMWindow(mozIDOMWindowProxy * *aHiddenPrivateDOMWindow) = 0;

  /* readonly attribute boolean applicationProvidedHiddenWindow; */
  NS_IMETHOD GetApplicationProvidedHiddenWindow(bool *aApplicationProvidedHiddenWindow) = 0;

  /* void registerTopLevelWindow (in nsIXULWindow aWindow); */
  NS_IMETHOD RegisterTopLevelWindow(nsIXULWindow *aWindow) = 0;

  /* void unregisterTopLevelWindow (in nsIXULWindow aWindow); */
  NS_IMETHOD UnregisterTopLevelWindow(nsIXULWindow *aWindow) = 0;

  /* [noscript] readonly attribute boolean hasHiddenPrivateWindow; */
  NS_IMETHOD GetHasHiddenPrivateWindow(bool *aHasHiddenPrivateWindow) = 0;

  /* bool startEventLoopLagTracking (); */
  NS_IMETHOD StartEventLoopLagTracking(bool *_retval) = 0;

  /* void stopEventLoopLagTracking (); */
  NS_IMETHOD StopEventLoopLagTracking(void) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIAppShellService, NS_IAPPSHELLSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIAPPSHELLSERVICE \
  NS_IMETHOD CreateTopLevelWindow(nsIXULWindow *aParent, nsIURI *aUrl, uint32_t aChromeMask, int32_t aInitialWidth, int32_t aInitialHeight, nsITabParent *aOpeningTab, mozIDOMWindowProxy *aOpenerWindow, nsIXULWindow * *_retval) override; \
  NS_IMETHOD CreateWindowlessBrowser(bool aIsChrome, nsIWindowlessBrowser * *_retval) override; \
  NS_IMETHOD CreateHiddenWindow(void) override; \
  NS_IMETHOD DestroyHiddenWindow(void) override; \
  NS_IMETHOD SetScreenId(uint32_t aScreenId) override; \
  NS_IMETHOD GetHiddenWindow(nsIXULWindow * *aHiddenWindow) override; \
  NS_IMETHOD GetHiddenDOMWindow(mozIDOMWindowProxy * *aHiddenDOMWindow) override; \
  NS_IMETHOD GetHiddenPrivateWindow(nsIXULWindow * *aHiddenPrivateWindow) override; \
  NS_IMETHOD GetHiddenPrivateDOMWindow(mozIDOMWindowProxy * *aHiddenPrivateDOMWindow) override; \
  NS_IMETHOD GetApplicationProvidedHiddenWindow(bool *aApplicationProvidedHiddenWindow) override; \
  NS_IMETHOD RegisterTopLevelWindow(nsIXULWindow *aWindow) override; \
  NS_IMETHOD UnregisterTopLevelWindow(nsIXULWindow *aWindow) override; \
  NS_IMETHOD GetHasHiddenPrivateWindow(bool *aHasHiddenPrivateWindow) override; \
  NS_IMETHOD StartEventLoopLagTracking(bool *_retval) override; \
  NS_IMETHOD StopEventLoopLagTracking(void) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIAPPSHELLSERVICE \
  NS_METHOD CreateTopLevelWindow(nsIXULWindow *aParent, nsIURI *aUrl, uint32_t aChromeMask, int32_t aInitialWidth, int32_t aInitialHeight, nsITabParent *aOpeningTab, mozIDOMWindowProxy *aOpenerWindow, nsIXULWindow * *_retval); \
  NS_METHOD CreateWindowlessBrowser(bool aIsChrome, nsIWindowlessBrowser * *_retval); \
  NS_METHOD CreateHiddenWindow(void); \
  NS_METHOD DestroyHiddenWindow(void); \
  NS_METHOD SetScreenId(uint32_t aScreenId); \
  NS_METHOD GetHiddenWindow(nsIXULWindow * *aHiddenWindow); \
  NS_METHOD GetHiddenDOMWindow(mozIDOMWindowProxy * *aHiddenDOMWindow); \
  NS_METHOD GetHiddenPrivateWindow(nsIXULWindow * *aHiddenPrivateWindow); \
  NS_METHOD GetHiddenPrivateDOMWindow(mozIDOMWindowProxy * *aHiddenPrivateDOMWindow); \
  NS_METHOD GetApplicationProvidedHiddenWindow(bool *aApplicationProvidedHiddenWindow); \
  NS_METHOD RegisterTopLevelWindow(nsIXULWindow *aWindow); \
  NS_METHOD UnregisterTopLevelWindow(nsIXULWindow *aWindow); \
  NS_METHOD GetHasHiddenPrivateWindow(bool *aHasHiddenPrivateWindow); \
  NS_METHOD StartEventLoopLagTracking(bool *_retval); \
  NS_METHOD StopEventLoopLagTracking(void); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIAPPSHELLSERVICE(_to) \
  NS_IMETHOD CreateTopLevelWindow(nsIXULWindow *aParent, nsIURI *aUrl, uint32_t aChromeMask, int32_t aInitialWidth, int32_t aInitialHeight, nsITabParent *aOpeningTab, mozIDOMWindowProxy *aOpenerWindow, nsIXULWindow * *_retval) override { return _to CreateTopLevelWindow(aParent, aUrl, aChromeMask, aInitialWidth, aInitialHeight, aOpeningTab, aOpenerWindow, _retval); } \
  NS_IMETHOD CreateWindowlessBrowser(bool aIsChrome, nsIWindowlessBrowser * *_retval) override { return _to CreateWindowlessBrowser(aIsChrome, _retval); } \
  NS_IMETHOD CreateHiddenWindow(void) override { return _to CreateHiddenWindow(); } \
  NS_IMETHOD DestroyHiddenWindow(void) override { return _to DestroyHiddenWindow(); } \
  NS_IMETHOD SetScreenId(uint32_t aScreenId) override { return _to SetScreenId(aScreenId); } \
  NS_IMETHOD GetHiddenWindow(nsIXULWindow * *aHiddenWindow) override { return _to GetHiddenWindow(aHiddenWindow); } \
  NS_IMETHOD GetHiddenDOMWindow(mozIDOMWindowProxy * *aHiddenDOMWindow) override { return _to GetHiddenDOMWindow(aHiddenDOMWindow); } \
  NS_IMETHOD GetHiddenPrivateWindow(nsIXULWindow * *aHiddenPrivateWindow) override { return _to GetHiddenPrivateWindow(aHiddenPrivateWindow); } \
  NS_IMETHOD GetHiddenPrivateDOMWindow(mozIDOMWindowProxy * *aHiddenPrivateDOMWindow) override { return _to GetHiddenPrivateDOMWindow(aHiddenPrivateDOMWindow); } \
  NS_IMETHOD GetApplicationProvidedHiddenWindow(bool *aApplicationProvidedHiddenWindow) override { return _to GetApplicationProvidedHiddenWindow(aApplicationProvidedHiddenWindow); } \
  NS_IMETHOD RegisterTopLevelWindow(nsIXULWindow *aWindow) override { return _to RegisterTopLevelWindow(aWindow); } \
  NS_IMETHOD UnregisterTopLevelWindow(nsIXULWindow *aWindow) override { return _to UnregisterTopLevelWindow(aWindow); } \
  NS_IMETHOD GetHasHiddenPrivateWindow(bool *aHasHiddenPrivateWindow) override { return _to GetHasHiddenPrivateWindow(aHasHiddenPrivateWindow); } \
  NS_IMETHOD StartEventLoopLagTracking(bool *_retval) override { return _to StartEventLoopLagTracking(_retval); } \
  NS_IMETHOD StopEventLoopLagTracking(void) override { return _to StopEventLoopLagTracking(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIAPPSHELLSERVICE(_to) \
  NS_IMETHOD CreateTopLevelWindow(nsIXULWindow *aParent, nsIURI *aUrl, uint32_t aChromeMask, int32_t aInitialWidth, int32_t aInitialHeight, nsITabParent *aOpeningTab, mozIDOMWindowProxy *aOpenerWindow, nsIXULWindow * *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->CreateTopLevelWindow(aParent, aUrl, aChromeMask, aInitialWidth, aInitialHeight, aOpeningTab, aOpenerWindow, _retval); } \
  NS_IMETHOD CreateWindowlessBrowser(bool aIsChrome, nsIWindowlessBrowser * *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->CreateWindowlessBrowser(aIsChrome, _retval); } \
  NS_IMETHOD CreateHiddenWindow(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->CreateHiddenWindow(); } \
  NS_IMETHOD DestroyHiddenWindow(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->DestroyHiddenWindow(); } \
  NS_IMETHOD SetScreenId(uint32_t aScreenId) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetScreenId(aScreenId); } \
  NS_IMETHOD GetHiddenWindow(nsIXULWindow * *aHiddenWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetHiddenWindow(aHiddenWindow); } \
  NS_IMETHOD GetHiddenDOMWindow(mozIDOMWindowProxy * *aHiddenDOMWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetHiddenDOMWindow(aHiddenDOMWindow); } \
  NS_IMETHOD GetHiddenPrivateWindow(nsIXULWindow * *aHiddenPrivateWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetHiddenPrivateWindow(aHiddenPrivateWindow); } \
  NS_IMETHOD GetHiddenPrivateDOMWindow(mozIDOMWindowProxy * *aHiddenPrivateDOMWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetHiddenPrivateDOMWindow(aHiddenPrivateDOMWindow); } \
  NS_IMETHOD GetApplicationProvidedHiddenWindow(bool *aApplicationProvidedHiddenWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetApplicationProvidedHiddenWindow(aApplicationProvidedHiddenWindow); } \
  NS_IMETHOD RegisterTopLevelWindow(nsIXULWindow *aWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RegisterTopLevelWindow(aWindow); } \
  NS_IMETHOD UnregisterTopLevelWindow(nsIXULWindow *aWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->UnregisterTopLevelWindow(aWindow); } \
  NS_IMETHOD GetHasHiddenPrivateWindow(bool *aHasHiddenPrivateWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetHasHiddenPrivateWindow(aHasHiddenPrivateWindow); } \
  NS_IMETHOD StartEventLoopLagTracking(bool *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->StartEventLoopLagTracking(_retval); } \
  NS_IMETHOD StopEventLoopLagTracking(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->StopEventLoopLagTracking(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsAppShellService : public nsIAppShellService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIAPPSHELLSERVICE

  nsAppShellService();

private:
  ~nsAppShellService();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsAppShellService, nsIAppShellService)

nsAppShellService::nsAppShellService()
{
  /* member initializers and constructor code */
}

nsAppShellService::~nsAppShellService()
{
  /* destructor code */
}

/* nsIXULWindow createTopLevelWindow (in nsIXULWindow aParent, in nsIURI aUrl, in uint32_t aChromeMask, in long aInitialWidth, in long aInitialHeight, in nsITabParent aOpeningTab, in mozIDOMWindowProxy aOpenerWindow); */
NS_IMETHODIMP nsAppShellService::CreateTopLevelWindow(nsIXULWindow *aParent, nsIURI *aUrl, uint32_t aChromeMask, int32_t aInitialWidth, int32_t aInitialHeight, nsITabParent *aOpeningTab, mozIDOMWindowProxy *aOpenerWindow, nsIXULWindow * *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIWindowlessBrowser createWindowlessBrowser ([optional] in bool aIsChrome); */
NS_IMETHODIMP nsAppShellService::CreateWindowlessBrowser(bool aIsChrome, nsIWindowlessBrowser * *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] void createHiddenWindow (); */
NS_IMETHODIMP nsAppShellService::CreateHiddenWindow()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void destroyHiddenWindow (); */
NS_IMETHODIMP nsAppShellService::DestroyHiddenWindow()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] void setScreenId (in uint32_t aScreenId); */
NS_IMETHODIMP nsAppShellService::SetScreenId(uint32_t aScreenId)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIXULWindow hiddenWindow; */
NS_IMETHODIMP nsAppShellService::GetHiddenWindow(nsIXULWindow * *aHiddenWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute mozIDOMWindowProxy hiddenDOMWindow; */
NS_IMETHODIMP nsAppShellService::GetHiddenDOMWindow(mozIDOMWindowProxy * *aHiddenDOMWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIXULWindow hiddenPrivateWindow; */
NS_IMETHODIMP nsAppShellService::GetHiddenPrivateWindow(nsIXULWindow * *aHiddenPrivateWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute mozIDOMWindowProxy hiddenPrivateDOMWindow; */
NS_IMETHODIMP nsAppShellService::GetHiddenPrivateDOMWindow(mozIDOMWindowProxy * *aHiddenPrivateDOMWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute boolean applicationProvidedHiddenWindow; */
NS_IMETHODIMP nsAppShellService::GetApplicationProvidedHiddenWindow(bool *aApplicationProvidedHiddenWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void registerTopLevelWindow (in nsIXULWindow aWindow); */
NS_IMETHODIMP nsAppShellService::RegisterTopLevelWindow(nsIXULWindow *aWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void unregisterTopLevelWindow (in nsIXULWindow aWindow); */
NS_IMETHODIMP nsAppShellService::UnregisterTopLevelWindow(nsIXULWindow *aWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] readonly attribute boolean hasHiddenPrivateWindow; */
NS_IMETHODIMP nsAppShellService::GetHasHiddenPrivateWindow(bool *aHasHiddenPrivateWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* bool startEventLoopLagTracking (); */
NS_IMETHODIMP nsAppShellService::StartEventLoopLagTracking(bool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void stopEventLoopLagTracking (); */
NS_IMETHODIMP nsAppShellService::StopEventLoopLagTracking()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIAppShellService_h__ */
