/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

/**
 * The SMIL TimeEvent interface.
 *
 * For more information please refer to:
 * http://www.w3.org/TR/SMIL/smil-timing.html#Events-TimeEvent
 * http://www.w3.org/TR/SVG/animate.html#InterfaceTimeEvent
 */

[builtinclass, uuid(b5e7fbac-f572-426c-9320-0ef7630f03c1)]
interface nsIDOMTimeEvent : nsIDOMEvent
{
  readonly attribute long detail;
};
