/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIQuotaResults.idl
 */

#ifndef __gen_nsIQuotaResults_h__
#define __gen_nsIQuotaResults_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIQuotaUsageResult */
#define NS_IQUOTAUSAGERESULT_IID_STR "d8c9328b-9aa8-4f5d-90e6-482de4a6d5b8"

#define NS_IQUOTAUSAGERESULT_IID \
  {0xd8c9328b, 0x9aa8, 0x4f5d, \
    { 0x90, 0xe6, 0x48, 0x2d, 0xe4, 0xa6, 0xd5, 0xb8 }}

class NS_NO_VTABLE nsIQuotaUsageResult : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IQUOTAUSAGERESULT_IID)

  /* readonly attribute ACString origin; */
  NS_IMETHOD GetOrigin(nsACString & aOrigin) = 0;

  /* readonly attribute boolean persisted; */
  NS_IMETHOD GetPersisted(bool *aPersisted) = 0;

  /* readonly attribute unsigned long long usage; */
  NS_IMETHOD GetUsage(uint64_t *aUsage) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIQuotaUsageResult, NS_IQUOTAUSAGERESULT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIQUOTAUSAGERESULT \
  NS_IMETHOD GetOrigin(nsACString & aOrigin) override; \
  NS_IMETHOD GetPersisted(bool *aPersisted) override; \
  NS_IMETHOD GetUsage(uint64_t *aUsage) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIQUOTAUSAGERESULT \
  NS_METHOD GetOrigin(nsACString & aOrigin); \
  NS_METHOD GetPersisted(bool *aPersisted); \
  NS_METHOD GetUsage(uint64_t *aUsage); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIQUOTAUSAGERESULT(_to) \
  NS_IMETHOD GetOrigin(nsACString & aOrigin) override { return _to GetOrigin(aOrigin); } \
  NS_IMETHOD GetPersisted(bool *aPersisted) override { return _to GetPersisted(aPersisted); } \
  NS_IMETHOD GetUsage(uint64_t *aUsage) override { return _to GetUsage(aUsage); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIQUOTAUSAGERESULT(_to) \
  NS_IMETHOD GetOrigin(nsACString & aOrigin) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetOrigin(aOrigin); } \
  NS_IMETHOD GetPersisted(bool *aPersisted) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetPersisted(aPersisted); } \
  NS_IMETHOD GetUsage(uint64_t *aUsage) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetUsage(aUsage); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsQuotaUsageResult : public nsIQuotaUsageResult
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIQUOTAUSAGERESULT

  nsQuotaUsageResult();

private:
  ~nsQuotaUsageResult();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsQuotaUsageResult, nsIQuotaUsageResult)

nsQuotaUsageResult::nsQuotaUsageResult()
{
  /* member initializers and constructor code */
}

nsQuotaUsageResult::~nsQuotaUsageResult()
{
  /* destructor code */
}

/* readonly attribute ACString origin; */
NS_IMETHODIMP nsQuotaUsageResult::GetOrigin(nsACString & aOrigin)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute boolean persisted; */
NS_IMETHODIMP nsQuotaUsageResult::GetPersisted(bool *aPersisted)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long long usage; */
NS_IMETHODIMP nsQuotaUsageResult::GetUsage(uint64_t *aUsage)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsIQuotaOriginUsageResult */
#define NS_IQUOTAORIGINUSAGERESULT_IID_STR "96df03d2-116a-493f-bb0b-118c212a6b32"

#define NS_IQUOTAORIGINUSAGERESULT_IID \
  {0x96df03d2, 0x116a, 0x493f, \
    { 0xbb, 0x0b, 0x11, 0x8c, 0x21, 0x2a, 0x6b, 0x32 }}

class NS_NO_VTABLE nsIQuotaOriginUsageResult : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IQUOTAORIGINUSAGERESULT_IID)

  /* readonly attribute unsigned long long usage; */
  NS_IMETHOD GetUsage(uint64_t *aUsage) = 0;

  /* readonly attribute unsigned long long fileUsage; */
  NS_IMETHOD GetFileUsage(uint64_t *aFileUsage) = 0;

  /* readonly attribute unsigned long long limit; */
  NS_IMETHOD GetLimit(uint64_t *aLimit) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIQuotaOriginUsageResult, NS_IQUOTAORIGINUSAGERESULT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIQUOTAORIGINUSAGERESULT \
  NS_IMETHOD GetUsage(uint64_t *aUsage) override; \
  NS_IMETHOD GetFileUsage(uint64_t *aFileUsage) override; \
  NS_IMETHOD GetLimit(uint64_t *aLimit) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIQUOTAORIGINUSAGERESULT \
  NS_METHOD GetUsage(uint64_t *aUsage); \
  NS_METHOD GetFileUsage(uint64_t *aFileUsage); \
  NS_METHOD GetLimit(uint64_t *aLimit); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIQUOTAORIGINUSAGERESULT(_to) \
  NS_IMETHOD GetUsage(uint64_t *aUsage) override { return _to GetUsage(aUsage); } \
  NS_IMETHOD GetFileUsage(uint64_t *aFileUsage) override { return _to GetFileUsage(aFileUsage); } \
  NS_IMETHOD GetLimit(uint64_t *aLimit) override { return _to GetLimit(aLimit); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIQUOTAORIGINUSAGERESULT(_to) \
  NS_IMETHOD GetUsage(uint64_t *aUsage) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetUsage(aUsage); } \
  NS_IMETHOD GetFileUsage(uint64_t *aFileUsage) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetFileUsage(aFileUsage); } \
  NS_IMETHOD GetLimit(uint64_t *aLimit) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLimit(aLimit); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsQuotaOriginUsageResult : public nsIQuotaOriginUsageResult
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIQUOTAORIGINUSAGERESULT

  nsQuotaOriginUsageResult();

private:
  ~nsQuotaOriginUsageResult();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsQuotaOriginUsageResult, nsIQuotaOriginUsageResult)

nsQuotaOriginUsageResult::nsQuotaOriginUsageResult()
{
  /* member initializers and constructor code */
}

nsQuotaOriginUsageResult::~nsQuotaOriginUsageResult()
{
  /* destructor code */
}

/* readonly attribute unsigned long long usage; */
NS_IMETHODIMP nsQuotaOriginUsageResult::GetUsage(uint64_t *aUsage)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long long fileUsage; */
NS_IMETHODIMP nsQuotaOriginUsageResult::GetFileUsage(uint64_t *aFileUsage)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long long limit; */
NS_IMETHODIMP nsQuotaOriginUsageResult::GetLimit(uint64_t *aLimit)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIQuotaResults_h__ */
